@@ -1,5 +1,5 @@
-from typing import Tuple, Union
 import math
+from typing import Tuple, Union
 
 import lab as B
 from plum import Dispatcher
@@ -17,28 +17,28 @@ _dispatch = Dispatcher()
 class MLP:
     def __init__(
         self,
-        dim_in: int,
-        dim_hidden: int,
-        dim_out: int,
-        num_layers: int,
+        in_dim: int,
+        layers: Tuple[int, ...],
+        out_dim: int,
         nonlinearity=None,
         dtype=None,
     ):
+
         # Default to ReLUs.
         if nonlinearity is None:
             nonlinearity = self.nn.ReLU()
 
         # Build layers.
-        if num_layers == 1:
-            layers = self.nn.Linear(dim_in, dim_out, dtype=dtype)
+        if len(layers) == 0:
+            self.net = self.nn.Linear(in_dim, out_dim, dtype=dtype)
         else:
-            layers = [self.nn.Linear(dim_in, dim_hidden, dtype=dtype)]
-            for _ in range(num_layers - 2):
-                layers.append(nonlinearity)
-                layers.append(self.nn.Linear(dim_hidden, dim_hidden, dtype=dtype))
-            layers.append(nonlinearity)
-            layers.append(self.nn.Linear(dim_hidden, dim_out, dtype=dtype))
-        self.net = self.nn.Sequential(*layers)
+            net = [self.nn.Linear(in_dim, layers[0], dtype=dtype)]
+            for i in range(1, len(layers)):
+                net.append(nonlinearity)
+                net.append(self.nn.Linear(layers[i - 1], layers[i], dtype=dtype))
+            net.append(nonlinearity)
+            net.append(self.nn.Linear(layers[-1], out_dim, dtype=dtype))
+            self.net = self.nn.Sequential(*net)
 
     def __call__(self, x):
         x = B.transpose(x)
@@ -57,8 +57,11 @@ class UNet:
         channels: Tuple[int, ...] = (8, 16, 16, 32, 32, 64),
         kernels: Union[int, Tuple[int, ...]] = 5,
         activations: Union[None, object, Tuple[object, ...]] = None,
+        resize_convs: bool = False,
+        resize_conv_interp_method: str = "nearest",
         dtype=None,
     ):
+
         # If `kernel` is an integer, repeat it for every layer.
         if not isinstance(kernels, (tuple, list)):
             kernels = (kernels,) * len(channels)
@@ -103,6 +106,7 @@ class UNet:
 
         Conv = getattr(self.nn, f"Conv{dim}d")
         ConvTranspose = getattr(self.nn, f"ConvTransposed{dim}d")
+        UpSampling = getattr(self.nn, f"UpSampling{dim}d")
         AvgPool = getattr(self.nn, f"AvgPool{dim}d")
 
         # Final linear layer:
@@ -152,9 +156,23 @@ class UNet:
                 # Add the skip connection.
                 return 2 * channels[i]
 
-        self.after_turn_layers = self.nn.ModuleList(
-            [
-                ConvTranspose(
+        def after_turn_layer(i):
+            if resize_convs:
+                return self.nn.Sequential(
+                    UpSampling(
+                        interp_method=resize_conv_interp_method,
+                        dtype=dtype,
+                    ),
+                    Conv(
+                        in_channels=get_num_in_channels(i),
+                        out_channels=((channels[0],) + channels)[i],
+                        kernel=self.kernels[i],
+                        stride=1,
+                        dtype=dtype,
+                    ),
+                )
+            else:
+                return ConvTranspose(
                     in_channels=get_num_in_channels(i),
                     out_channels=((channels[0],) + channels)[i],
                     kernel=self.kernels[i],
@@ -162,8 +180,9 @@ class UNet:
                     output_padding=1,
                     dtype=dtype,
                 )
-                for i in range(len(channels))
-            ]
+
+        self.after_turn_layers = self.nn.ModuleList(
+            [after_turn_layer(i) for i in range(len(channels))]
         )
 
     def __call__(self, x):
@@ -201,7 +220,8 @@ class ConvNet:
         dtype=None,
     ):
         activation = self.nn.ReLU()
-        # To make it a drop-in substitute for :class:`UNet`.
+
+        # Make it a drop-in substitute for :class:`UNet`.
         self.num_halving_layers = 0
 
         # Compute kernel size.
@@ -215,6 +235,7 @@ class ConvNet:
                 in_channels=in_channels,
                 out_channels=channels,
                 kernel=1,
+                dtype=dtype,
             ),
             activation,
         ]
@@ -226,11 +247,13 @@ class ConvNet:
                         out_channels=channels,
                         kernel=kernel,
                         groups=channels,
+                        dtype=dtype,
                     ),
                     Conv(
                         in_channels=channels,
                         out_channels=channels,
                         kernel=1,
+                        dtype=dtype,
                     ),
                     activation,
                 ]
@@ -240,6 +263,7 @@ class ConvNet:
                 in_channels=channels,
                 out_channels=out_channels,
                 kernel=1,
+                dtype=dtype,
             )
         )
         self.conv_net = self.nn.Sequential(*layers)
